"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseToken = void 0;
const ethers_1 = require("ethers");
const web3_bridge_1 = require("../bridge/web3-bridge");
const base_bridge_1 = require("../bridge/base-bridge");
class BaseToken {
    constructor(bridgeParams, contractABI) {
        if (!bridgeParams.chain) {
            throw new Error('Token must contain chain');
        }
        if (!bridgeParams.contractAddress) {
            throw new Error('Token must contain contract address');
        }
        this.bridgeParams = { ...bridgeParams, externalWalletId: bridgeParams.contractAddress };
        this.web3Bridge = new web3_bridge_1.Web3Bridge(bridgeParams);
        this.contractABI = contractABI;
        this.contract = new ethers_1.ethers.Contract(this.bridgeParams.contractAddress, JSON.stringify(this.contractABI), ethers_1.ethers.getDefaultProvider(this.bridgeParams.chain));
        this._allFunctions = this.contract.functions;
    }
    /**
     * Use this to build a transaction which will then be sent to fireblocks to get signed.
     * @param abiName - abi function name
     * @param args - function params
     */
    async buildTransaction(abiName, ...args) {
        return await this.contract.populateTransaction[abiName].call(this, ...args);
    }
    /**
     * Use this to execute a function that needs no signing, e.g. supportsInterface. Basically a "read" function
     * @param abiName
     * @param args
     */
    async callView(abiName, ...args) {
        var _a;
        try {
            const response = await this.contract.functions[abiName].call(this, ...args);
            return (_a = response[0]) !== null && _a !== void 0 ? _a : response;
        }
        catch (e) {
            throw new Error(e);
        }
    }
    /**
     * Fetching Fireblocks default address using chain
     */
    async getAddress() {
        var _a;
        if (!this.address) {
            const res = await this.bridgeParams.fireblocksApiClient.getDepositAddresses(this.bridgeParams.vaultAccountId, base_bridge_1.CHAIN_TO_ASSET_ID[this.bridgeParams.chain]);
            this.address = (res === null || res === void 0 ? void 0 : res.length) > 0 ? (_a = res.pop()) === null || _a === void 0 ? void 0 : _a.address : null;
        }
        return this.address;
    }
    /**
     * Submit transaction using Fireblocks infrastructure
     * @param transactionData
     * @param notes - (Optional) Add a note to the transaction
     */
    submitTransaction(transactionData, notes) {
        return this.web3Bridge.sendTransaction(transactionData, notes);
    }
}
exports.BaseToken = BaseToken;
