"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ERC1155 = void 0;
const base_token_1 = require("./base-token");
const base_abis_1 = require("../constants/base-abis");
const web3_1 = __importDefault(require("web3"));
class ERC1155 extends base_token_1.BaseToken {
    constructor(bridgeParams) {
        super(bridgeParams, base_abis_1.ERC1155_ABI);
    }
    /** Payable **/
    /**
     * Length of token_ids and values must match. Moreover, the value of each token (at position x at [values]) to be
     * sent must match the same position at [token_ids] (position x). For example:
     * token_ids [3, 6, 10] and values [10, 20, 30] will send:
     *         - 10 tokens of token id 3
     *         - 20 tokens of token id 6
     *         - 30 tokens of token id 10
     * @param fromAddress - The sender of the address token.
     * @param toAddress - The receiver of the address token.
     * @param tokenIds - A list of token ids to transfer from sender to receiver.
     * @param values
     * @param data
     * @param note
     */
    async safeBatchTransferFrom(toAddress, tokenIds, values, data = null, note = "", fromAddress) {
        const checkedFromAddress = web3_1.default.utils.toChecksumAddress(fromAddress || await this.getAddress());
        const checkedToAddress = web3_1.default.utils.toChecksumAddress(toAddress);
        if ((tokenIds === null || tokenIds === void 0 ? void 0 : tokenIds.length) != (values === null || values === void 0 ? void 0 : values.length)) {
            throw new Error('Length of token_ids and values must match!');
        }
        console.log('safeBatchTransferFrom', checkedFromAddress, checkedToAddress, tokenIds, values, data || new Uint8Array());
        let transactionData;
        transactionData = await this.buildTransaction("safeTransferFrom", checkedFromAddress, checkedToAddress, tokenIds, values, data || new Uint8Array());
        console.log('transactionData', transactionData);
        return this.submitTransaction(transactionData, note);
    }
    /**
     *
     * @param fromAddress - The sender of the address token.
     * @param toAddress - The receiver of the address token.
     * @param tokenId - The ID of the sender address token.
     * @param data - Send additional data (bytes) only if required by contract.
     * @param note
     */
    async safeTransferFrom(toAddress, tokenId, data = null, note = "", fromAddress) {
        const checkedFromAddress = web3_1.default.utils.toChecksumAddress(fromAddress || await this.getAddress());
        const checkedToAddress = web3_1.default.utils.toChecksumAddress(toAddress);
        let transactionData;
        if (data) {
            transactionData = await this.buildTransaction("safeTransferFrom", checkedFromAddress, checkedToAddress, tokenId, data);
        }
        else {
            transactionData = await this.buildTransaction("safeTransferFrom", checkedFromAddress, checkedToAddress, tokenId);
        }
        return this.submitTransaction(transactionData, note);
    }
    /**
     * Provide an operator with approval permission or revoke them
     * @param operatorAddress - Approved operator address
     * @param isApproved - True to permit, False to revoke
     * @param notes - (Optional) Add a note to the transaction
     */
    async setApprovalForAll(operatorAddress, isApproved, notes) {
        const checkedOperatorAddress = web3_1.default.utils.toChecksumAddress(operatorAddress);
        const transactionData = await this.buildTransaction("setApprovalForAll", checkedOperatorAddress, isApproved);
        return this.submitTransaction(transactionData, notes);
    }
    /** Views **/
    /**
     * Checks if contract supports a certain interface.
     * @param interfaceId - (Optional) The interface id. "0x80ac58cd" is ERC721 interface id. "0xd9b67a26" is ERC1155
     */
    supportsInterface(interfaceId = '0xd9b67a26') {
        return this.callView("supportsInterface", interfaceId);
    }
    /**
     *
     * @param tokenId
     * @param ownerAddress
     */
    async balanceOf(tokenId, ownerAddress) {
        const ownerCheckedAddress = web3_1.default.utils.toChecksumAddress(ownerAddress || await this.getAddress());
        return this.callView("balanceOf", ownerCheckedAddress, tokenId);
    }
    /**
     *
     * @param ownersList -  A list of addresses
     * @param idList - A list of token Ids
     */
    async balanceOfBatch(idList, ownersList) {
        const checkedAddresses = (ownersList === null || ownersList === void 0 ? void 0 : ownersList.map(address => web3_1.default.utils.toChecksumAddress(address))) ||
            idList.map(async (res) => web3_1.default.utils.toChecksumAddress(await this.getAddress()));
        return this.callView("balanceOfBatch", checkedAddresses, idList);
    }
    /**
     * Validates whether an operator has been approved by a stated owner.
     * @param ownerAddress - Address of the owner.
     * @param operatorAddress - Address of the meant to be operator.
     * return: True whether operator is approved, False otherwise
     */
    async isApprovedForAll(operatorAddress, ownerAddress) {
        const ownerCheckedAddress = web3_1.default.utils.toChecksumAddress(ownerAddress || await this.getAddress());
        const operatorCheckedAddress = web3_1.default.utils.toChecksumAddress(operatorAddress);
        return this.callView("isApprovedForAll", ownerCheckedAddress, operatorCheckedAddress);
    }
    /**
     * Returns Token URI
     * @param tokenId
     */
    uri(tokenId) {
        return this.callView("uri", tokenId);
    }
}
exports.ERC1155 = ERC1155;
